#
# = confile.rb: модуль для чтения и парсинга конфигурационного файла avreg.conf
#

module AVRegUnlinkConf
   module ConfFile
      COMMENTS_REGX    = /^\s*[;#]/
      INGROUPS_REGX    = /^\s*([^\s=]+)[\s=]*\{\s*$/
      LOSEGROUPS_REGX  = /.*\}\s*$/
      # PARVAL_PAIR_REGX = /^\s*([^\s=]+)[\s="\']*(.*?)*["\']*\s*$/
      PARVAL_PAIR_REGX = /\A\s*([\w-]+)\s*=?\s*(["']?)\s*(.+?)\s*\2\s*\Z/
      def self.parse( opts_parser, sinlge_conf, path, section)
         accepted_params = 0
         conf_name = File.basename(path)
         skip_section = false

         f = File.open(path,'r:UTF-8')
         f.each_line do |line|
            line.chomp!

            next if line.empty?

            next if line =~ COMMENTS_REGX

            if line =~ INGROUPS_REGX
               # begin section
               if section.nil? or 0 != $1.casecmp(section)
                  skip_section = true
               end
               next
            end

            if line =~ LOSEGROUPS_REGX
               skip_section = false
               next
            end

            next if skip_section

            if line =~ PARVAL_PAIR_REGX
               param=$1
               value=$3
               # нашли параметр
               if 0 == param.casecmp('include')
                  begin
                     accepted_params += ConfFile.parse( opts_parser, sinlge_conf, value, nil)
                  rescue Errno::ENOENT
                     $log.alert "config file `#{value}' not exist, it included from #{conf_name}:#{f.lineno}"
                     raise
                  end
               else
                  # допустим ли вообще такой параметр?
                  # и не определён ли уже командной строкой (больший приоритет по сравнению с конф. файлом)
                  begin
                     if param == 'devlist'
                        sinlge_conf.parse_cameras_range(value)
                     else
                        if sinlge_conf.include?(param) or sinlge_conf.include?(param.sub(/^no-/, ''))
                           if value.nil? or value.empty? then
                              # --[no]-flag
                              opts_parser.parse!(["--#{param}"])
                           else
                              opts_parser.parse!(["--#{param}=#{value}"])
                           end
                        end
                     end
                  rescue ReadOnlyError
                     next
                  rescue OptionParser::ParseError, OptionParser::InvalidArgument
                     $log.alert "#{conf_name}:#{f.lineno} invalid value `#{value}' for `#{param}' option"
                     raise
                  else
                     accepted_params += 1
                  end
               end
            end

            # puts line
         end # foreach
         f.close unless f.nil?
         return accepted_params
      end # parse
   end # module ConfFile
end # module
