#
# = logger.rb: модуль с классом для вывода сообщений.
#

require 'syslog'

module AVRegUnlinkLogger

   # Класс с функциями для вывода сообщений в терминал и syslog.
   class Logger
      include Syslog

      def initialize(prog_name)
         @base_name = prog_name
         @profile = nil
         @syslog = Syslog.open(@base_name, 0, Syslog::LOG_DAEMON)
      end


      def emerg(message)
         @syslog.emerg( _('EMERG') + ': %s', message )
         print_prefix($stderr)
         $stderr.puts( _('EMERG') +  ': '  + message )
      end

      def alert(message)
         @syslog.alert( _('ALERT') + ': %s', message )
         print_prefix($stderr)
         $stderr.puts(  _('ALERT') +  ': '  + message )
      end

      def crit(message)
         @syslog.crit( _('CRITICAL') + ': %s', message )
         print_prefix($stderr)
         $stderr.puts( _('CRITICAL') +  ': '  + message )
      end

      def err(message)
         @syslog.err(  _('ERROR') + ': %s', message )
         print_prefix($stderr)
         $stderr.puts( _('ERROR') +  ': '  + message )
      end

      def warning(message)
         @syslog.warning( _('WARNING') + ': %s', message )
         print_prefix($stderr)
         $stderr.puts(   _('WARNING') +  ': '  + message )
      end

      def notice(message)
         @syslog.notice( '%s', message)
         print_prefix($stdout)
         $stdout.puts(message)
      end

      def info(message)
         return unless $verbose >= 1
         @syslog.info( '%s', message)
         print_prefix($stdout)
         $stdout.puts(message)
      end

      def debug(message)
         return unless $verbose >= 2
         @syslog.debug( '%s', message)
         print_prefix($stdout)
         $stdout.puts(message)
      end

      def close
         @syslog if not @syslog and @syslog.opened?
      end

      def reopen(profile_name)
         return if not profile_name or profile_name == ''
         @profile = profile_name
         @syslog.reopen("#@base_name-#@profile", 0, Syslog::LOG_DAEMON)
      end

      private
      def print_prefix(io)
         io.print("#{@profile}: ") if @profile and @profile != ''
      end
   end
end
